@@ -143,11 +143,11 @@ the exception of those in mlab.py provided by matplotlib.
 from __future__ import generators
 
 
-__version__  = '0.87'
-__revision__ = '$Revision: 1.122 $'
-__date__     = '$Date: 2006/02/22 14:33:56 $'
+__version__  = '0.87.5'
+__revision__ = '$Revision: 2761 $'
+__date__     = '$Date: 2006-09-05 09:37:04 -0400 (Tue, 05 Sep 2006) $'
 
-import sys, os, warnings, shutil, md5
+import md5, os, re, shutil, sys, warnings
 import distutils.sysconfig
 
 # Needed for toolkit setuptools support
@@ -338,7 +338,7 @@ def _get_data_path():
     if os.environ.has_key('MATPLOTLIBDATA'):
         path = os.environ['MATPLOTLIBDATA']
         if os.path.isdir(path): return path
-    
+
     path = os.sep.join([os.path.dirname(__file__), 'mpl-data'])
     if os.path.isdir(path): return path
 
@@ -347,9 +347,9 @@ def _get_data_path():
     import matplotlib.afm
     path = os.sep.join([os.path.dirname(matplotlib.afm.__file__), 'mpl-data'])
     if os.path.isdir(path): return path
-    
+
     # py2exe zips pure python, so still need special check
-    if (sys.platform=='win32' or sys.platform=='darwin') and sys.frozen:
+    if getattr(sys,'frozen',None):
         path = os.path.join(os.path.split(sys.path[0])[0], 'matplotlibdata')
         if os.path.isdir(path):  return path
         else:
@@ -363,11 +363,11 @@ get_data_path = verbose.wrap('matplotlib data path %s', _get_data_path, always=F
 
 def get_py2exe_datafiles():
     import glob
-    
-    mplfiles = glob.glob(os.sep.join([matplotlib.get_data_path(), '*']))
+
+    mplfiles = glob.glob(os.sep.join([get_data_path(), '*']))
     # Need to explicitly remove cocoa_agg files or py2exe complains
-    mplfiles.remove(os.sep.join([matplotlib.get_data_path(), 'Matplotlib.nib']))
-    
+    mplfiles.remove(os.sep.join([get_data_path(), 'Matplotlib.nib']))
+
     return ('matplotlibdata', mplfiles)
 
 def checkdep_dvipng():
@@ -407,14 +407,12 @@ def checkdep_ps2eps():
 
 def checkdep_tex():
     try:
-        stdin, stdout = os.popen4('tex -v')
+        stdin, stdout = os.popen4('tex -version')
         line = stdout.readlines()[0]
-        v = 0.0 # no version installed until we find one...
-        for potential_version_numstr in line.split():
-            if potential_version_numstr.startswith('3.1'):
-                v = potential_version_numstr
-                float(v)
-                break # found version number
+        pattern = '3\.1\d+'
+        match = re.search(pattern, line)
+        v = match.group(0)
+        float(v)
         return v
     except (IndexError, ValueError):
         return None
@@ -422,8 +420,9 @@ def checkdep_tex():
 def checkdep_xpdf():
     try:
         stdin, stdout = os.popen4('xpdf -v')
-        line = stdout.readlines()[0]
-        v = line.split()[-1]
+        for line in stdout.readlines():
+            if 'version' in line:
+                v = line.split()[-1]
         float(v)
         return v
     except (IndexError, ValueError):
@@ -465,8 +464,19 @@ def validate_int(s):
     except ValueError:
         raise ValueError('Could not convert "%s" to int' % s)
 
+def validate_backend(s, fail_on_err = True):
+    s=s.lower()
+    backends = ['Agg2', 'Agg', 'Aqt', 'Cairo', 'CocoaAgg', 'EMF', 'GD', 'GDK',
+                'GTK', 'GTKAgg', 'GTKCairo', 'FltkAgg', 'Paint', 'Pdf', 'PS',
+                'QtAgg', 'Qt4Agg', 'SVG', 'Template', 'TkAgg', 'WX', 'WXAgg']
+    for i in backends:
+        if s == i.lower(): return i
+    if fail_on_err: raise ValueError('Backend must be %s, or %s'% \
+                        ', '.join((backends[:-1], backends[-1])))
+    else: return None
+
 def validate_numerix(s):
-    'return "Numeric" or "Numarray" or raise'
+    'return "Numeric" or "numarray" or "numpy" or raise'
     sl = s.lower()
     if sl=='numeric': return 'Numeric'
     elif sl=='numarray': return 'numarray'
@@ -512,11 +522,8 @@ def validate_color(s):
         except ValueError:
             raise ValueError('Could not convert all entries "%s" to floats'%s)
 
-    if s.replace('.', '').isdigit(): # looks like scalar
-        try: return float(s)
-        except ValueError:
-            raise ValueError('Could not convert "%s" to float' % s)
-
+    if s.replace('.', '').isdigit(): # looks like scalar (grayscale)
+        return s
 
     if len(s)==6 and s.isalnum(): # looks like hex
         return '#' + s
@@ -552,6 +559,14 @@ class ValidateInStrings:
 
 validate_orientation = ValidateInStrings(['landscape', 'portrait'])
 
+def validate_aspect(s):
+    if s in ('auto', 'equal'):
+        return s
+    try:
+        return float(s)
+    except ValueError:
+        raise ValueError('not a valid aspect specification')
+
 def validate_fontsize(s):
     if s.lower() in ['xx-small', 'x-small', 'small', 'medium', 'large', 'x-large',
              'xx-large', 'smaller', 'larger']:
@@ -582,10 +597,9 @@ def validate_verbose_fileo(s):
 
 
 validate_ps_papersize = ValidateInStrings([
-        'executive', 'letter', 'legal', 'ledger',
+        'auto', 'letter', 'legal', 'ledger',
         'a0', 'a1', 'a2','a3', 'a4', 'a5', 'a6', 'a7', 'a8', 'a9', 'a10',
-        'b0', 'b1', 'b2', 'b3', 'b4', 'b5', 'b6',
-        'c0', 'c1', 'c2', 'c3', 'c4', 'c5', 'c6'
+        'b0', 'b1', 'b2', 'b3', 'b4', 'b5', 'b6', 'b7', 'b8', 'b9', 'b10',
         ], ignorecase=True)
 
 def validate_ps_distiller(s):
@@ -598,7 +612,7 @@ def validate_ps_distiller(s):
     elif s in ('ghostscript', 'xpdf'):
         flag = True
         gs_req = '7.07'
-        gs_sugg = '8.16'
+        gs_sugg = '7.07'
         gs_v = checkdep_ghostscript()
         if compare_versions(gs_v, gs_sugg): pass
         elif compare_versions(gs_v, gs_req):
@@ -619,12 +633,12 @@ unless ghostscript-%s or later is installed on your system'% gs_req)
                 warnings.warn('matplotlibrc ps.usedistiller can not be set to \
 xpdf unless xpdf-%s or later is installed on your system' % xpdf_req)
 
-            ps2eps_v = checkdep_ps2eps()
-            if compare_versions(ps2eps_v, ps2eps_req): pass
-            else:
-                flag = False
-                warnings.warn('matplotlibrc ps.usedistiller can not be set to xpdf \
-unless ps2eps-%s or later is installed on your system' % ps2eps_req)
+##            ps2eps_v = checkdep_ps2eps()
+##            if compare_versions(ps2eps_v, ps2eps_req): pass
+##            else:
+##                flag = False
+##                warnings.warn('matplotlibrc ps.usedistiller can not be set to xpdf \
+##unless ps2eps-%s or later is installed on your system' % ps2eps_req)
 
         if flag: return s
         else: return None
@@ -637,7 +651,7 @@ def validate_usetex(s):
     if bl:
         tex_req = '3.1415'
         gs_req = '7.07'
-        gs_sugg = '8.16'
+        gs_sugg = '7.07'
         dvipng_req = '1.5'
         flag = True
 
@@ -704,7 +718,7 @@ class ValidateInterval:
 
 # a map from key -> value, converter
 defaultParams = {
-    'backend'           : ['GTK', str],
+    'backend'           : ['GTK', validate_backend],
     'numerix'           : ['Numeric', validate_numerix],
     'toolbar'           : ['toolbar2', validate_toolbar],
     'datapath'          : [get_data_path(), validate_path_exists],
@@ -761,13 +775,16 @@ defaultParams = {
     'text.fontsize'     : ['medium', validate_fontsize],
 
 
-    'image.aspect' : ['free', str],  # free| preserve
+    'image.aspect' : ['equal', validate_aspect],  # equal, auto, a number
     'image.interpolation'  : ['bilinear', str],
     'image.cmap'   : ['gray', str],        # one of gray, jet, etc
     'image.lut'    : [256, validate_int],  # lookup table
     'image.origin'    : ['upper', str],  # lookup table
 
+    'contour.negative_linestyle' : [(6.0,6.0), validate_nseq_float(2)],
+
     # axes props
+    'axes.axisbelow'         : [False, validate_bool],
     'axes.hold'         : [True, validate_bool],
     'axes.facecolor'    : ['w', validate_color],    # background color; white
     'axes.edgecolor'    : ['k', validate_color],    # edge color; black
@@ -780,16 +797,19 @@ defaultParams = {
     'polaraxes.grid'         : [True, validate_bool],   # display polar grid or not
 
     #legend properties
-    'legend.isaxes'	:	[True,validate_bool],
-    'legend.numpoints' 	:	[ 4,validate_int],      # the number of points in the legend line
+    'legend.isaxes'    :       [True,validate_bool],
+    'legend.numpoints'         :       [ 4,validate_int],      # the number of points in the legend line
     'legend.fontsize' : ["small",validate_fontsize],
-    'legend.pad' 	:	[ 0.2, validate_float],         # the fractional whitespace inside the legend border
-    'legend.markerscale' 	:	[ 0.6, validate_float],    # the relative size of legend markers vs. original
+    'legend.pad'       :       [ 0.2, validate_float],         # the fractional whitespace inside the legend border
+    'legend.markerscale'       :       [ 0.6, validate_float],    # the relative size of legend markers vs. original
+
+
     # the following dimensions are in axes coords
-    'legend.labelsep' 	:	[ 0.005, validate_float],    # the vertical space between the legend entries
-    'legend.handlelen' 	:	[ 0.05, validate_float],  # the length of the legend lines
-    'legend.handletextsep' 	:	[ 0.02, validate_float], # the space between the legend line and legend text
-    'legend.axespad' 	:	[ 0.02, validate_float], # the border between the axes and legend edge
+    'legend.labelsep'  :       [ 0.005, validate_float],    # the vertical space between the legend entries
+    'legend.handlelen'         :       [ 0.05, validate_float],  # the length of the legend lines
+    'legend.handletextsep'     :       [ 0.02, validate_float], # the space between the legend line and legend text
+    'legend.axespad'   :       [ 0.02, validate_float], # the border between the axes and legend edge
+
     'legend.shadow' : [ False, validate_bool ],
 
 
@@ -819,7 +839,7 @@ defaultParams = {
     # figure size in inches: width by height
     'figure.figsize'    : [ (8,6), validate_nseq_float(2)],
     'figure.dpi'        : [ 80, validate_float],   # DPI
-    'figure.facecolor'  : [ 0.75, validate_color], # facecolor; scalar gray
+    'figure.facecolor'  : [ '0.75', validate_color], # facecolor; scalar gray
     'figure.edgecolor'  : [ 'w', validate_color],  # edgecolor; white
 
     'figure.subplot.left'   : [0.125, ValidateInterval(0, 1, closedmin=False, closedmax=False)],
@@ -841,8 +861,19 @@ defaultParams = {
     'ps.useafm'   : [ False, validate_bool],  # Set PYTHONINSPECT
     'ps.usedistiller'   : [ False, validate_ps_distiller],  # use ghostscript or xpdf to distill ps output
     'ps.distiller.res'  : [6000, validate_int],       # dpi
+    'pdf.compression'   : [6, validate_int],            # compression level from 0 to 9; 0 to disable
     'plugins.directory' : ['.matplotlib_plugins', str], # where plugin directory is locate
 
+    # mathtext settings
+    'mathtext.mathtext2'  :   [False, validate_bool], # Needed to enable Unicode
+    # fonts used by mathtext. These ship with matplotlib
+    'mathtext.rm'       :   ['cmr10.ttf', str], # Roman (normal)
+    'mathtext.it'       :   ['cmmi10.ttf', str], # Italic
+    'mathtext.tt'       :   ['cmtt10.ttf', str],  # Typewriter (monospaced)
+    'mathtext.mit'       :   ['cmmi10.ttf', str], # Math italic
+    'mathtext.cal'      :   ['cmsy10.ttf', str], # Caligraphic
+    'mathtext.nonascii' :   ['cmex10.ttf', str], # All other nonascii fonts
+
     }
 
 
@@ -908,7 +939,12 @@ def validate_key(key, val, line, cnt, fname, fail_on_error):
         key = alt
 
     if not defaultParams.has_key(key):
-        warnings.warn('Bad key "%s" on line %d in %s' % (key, cnt, fname))
+        print >> sys.stderr, """\
+Bad key "%s" on line %d in
+%s.
+You probably need to get an updated matplotlibrc file from
+http://matplotlib.sf.net/matplotlibrc or from the matplotlib source
+distribution""" % (key, cnt, fname)
         return None
 
     default, converter =  defaultParams[key]
@@ -966,12 +1002,12 @@ def rc_params(fail_on_error=False):
         except KeyError: continue
         else:
             cval = validate_key(key, val, line, cnt, fname, fail_on_error)
-            if cval: defaultParams[key][0] = cval
+            if cval is not None: defaultParams[key][0] = cval
     while len(rc_temp) > 0:
         key, (val, line, cnt) = rc_temp.popitem()
-        
+
         cval = validate_key(key, val, line, cnt, fname, fail_on_error)
-        if cval: defaultParams[key][0] = cval
+        if cval is not None: defaultParams[key][0] = cval
         else: continue
 
     # strip the converter funcs and return
@@ -1057,7 +1093,6 @@ def rc(group, **kwargs):
 
             rcParams[key] = v
 
-
 def rcdefaults():
     """
     Restore the default rc params - the ones that were created at
@@ -1065,47 +1100,24 @@ def rcdefaults():
     """
     rcParams.update(rcParamsDefault)
 
-
-
-
-
-
-
-
-
 # Now allow command line to override
 
 # Allow command line access to the backend with -d (matlab compatible
 # flag)
 
-_knownBackends = {
-    'Agg2':1, 'Agg':1, 'Cairo':1, 'CocoaAgg':1, 'FltkAgg':1, 'GD':1, 'GDK':1,
-    'GTK':1, 'GTKAgg':1, 'GTKCairo':1, 'Paint':1, 'PS':1, 'LaTeX':1, 'QtAgg':1,
-    'SVG':1, 'EMF':1, 'Template':1, 'TkAgg':1, 'WX':1, 'WXAgg':1, 'WXGLAgg':1,}
-
-
-known = _knownBackends.keys()
 for s in sys.argv[1:]:
     if s.startswith('-d'):  # look for a -d flag
-        name = s[2:].strip()
+        name = validate_backend(s[2:].strip(), fail_on_err = False)
         # we don't want to assume all -d flags are backends, eg -debug
-        if name in  known:
+        if name:
             rcParams['backend'] = name
             break
 
-
-
 def use(arg):
     """
-    Set the matplotlib backend to one of the _knownBackends
+    Set the matplotlib backend to one of the known backends
     """
-
-    if not _knownBackends.has_key(arg):
-        raise ValueError('unrecognized backend %s.\n' % arg +\
-              'Use one of %s' % ', '.join( _knownBackends.keys() ))
-
-
-    rcParams['backend'] = arg
+    rcParams['backend'] = validate_backend(arg)
 
 def get_backend():
     return rcParams['backend']
