@@ -16,8 +16,8 @@ any results.
 Copy this to backend_xxx.py and replace all instances of 'template'
 with 'xxx'.  Then implement the class methods and functions below, and
 add 'xxx' to the switchyard in matplotlib/backends/__init__.py and
-'xxx' to the _knownBackends dict in matplotlib/__init__.py and you're
-off.  You can use your backend with
+'xxx' to the backends list in the validate_backend methon in
+matplotlib/__init__.py and you're off.  You can use your backend with
 
   import matplotlib
   matplotlib.use('xxx')
@@ -35,19 +35,11 @@ The files that are most relevant to backend_writers are
   
 Naming Conventions
 
-  * classes MixedUpperCase
+  * classes Upper or MixedUpperCase
 
-  * varables lowerUpper
+  * varables lower or lowerUpper
 
-  * functions underscore_separated
-
-REQUIREMENTS
-
-  matplotlib requires python2.2 and Numeric, and I don't yet want to
-  make python2.3 a requirement.  I provide the Python Cookbook version
-  of enumerate in cbook.py and define the constants True and False if
-  version <=2.3.  Of course as a backend writer, you are free to make
-  additional requirements, but the less required the better.
+  * functions lower or underscore_separated
 
 """
 
@@ -69,7 +61,7 @@ class RendererTemplate(RendererBase):
     writing a new backend. Refer to backend_bases.RendererBase for
     documentation of the classes methods.
     """
-    def draw_arc(self, gcEdge, rgbFace, x, y, width, height, angle1, angle2):
+    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2, rotation):
         pass
     
     def draw_image(self, x, y, im, bbox):
@@ -174,8 +166,8 @@ def new_figure_manager(num, *args, **kwargs):
     # do it -- see backend_wx, backend_wxagg and backend_tkagg for
     # examples.  Not all GUIs require explicit instantiation of a
     # main-level app (egg backend_gtk, backend_gtkagg) for pylab
-
-    thisFig = Figure(*args, **kwargs)
+    FigureClass = kwargs.pop('FigureClass', Figure)
+    thisFig = FigureClass(*args, **kwargs)
     canvas = FigureCanvasTemplate(thisFig)
     manager = FigureManagerTemplate(canvas, num)
     return manager
@@ -205,7 +197,7 @@ class FigureCanvasTemplate(FigureCanvasBase):
         self.figure.draw(renderer)
         
     def print_figure(self, filename, dpi=150, facecolor='w', edgecolor='w',
-                     orientation='portrait'):
+                     orientation='portrait', **kwargs):
         """
         Render the figure to hardcopy. Set the figure patch face and edge
         colors.  This is useful because some of the GUIs have a gray figure
