@@ -92,7 +92,7 @@ Examples which work on this release:
  (3) - Clipping seems to be broken.
 """
 
-cvs_id = '$Id: backend_wx.py,v 1.28 2005/05/17 21:04:09 newville Exp $'
+cvs_id = '$Id: backend_wx.py 2728 2006-08-29 23:16:35Z cmoad $'
 
 import sys, os, os.path, math, StringIO
 
@@ -157,7 +157,7 @@ from matplotlib.artist import Artist
 from matplotlib.cbook import exception_to_str
 from matplotlib.figure import Figure
 from matplotlib.text import _process_text_args, Text
-
+from matplotlib.widgets import SubplotTool
 from matplotlib import rcParams
 
 ##import wx
@@ -823,6 +823,27 @@ The current aspect ration will be kept."""
         dlg.Destroy()
         return
 
+    def Printer_Setup2(self, event=None):
+        """set up figure for printing.  Using the standard wx Printer
+        Setup Dialog. """
+
+        if hasattr(self, 'printerData'):
+            data = wx.PageSetupDialogData()
+            data.SetPrintData(self.printerData)
+        else:
+            data = wx.PageSetupDialogData()
+        data.SetMarginTopLeft( (15, 15) )
+        data.SetMarginBottomRight( (15, 15) )
+
+        dlg = wx.PageSetupDialog(self, data)
+
+        if dlg.ShowModal() == wx.ID_OK:
+            data = dlg.GetPageSetupData()
+            tl = data.GetMarginTopLeft()
+            br = data.GetMarginBottomRight()
+        self.printerData = wx.PrintData(data.GetPrintData())
+        dlg.Destroy()
+
     def Printer_Preview(self, event=None):
         """ generate Print Preview with wx Print mechanism"""
         po1  = PrintoutWx(self, width=self.printer_width,
@@ -869,7 +890,7 @@ The current aspect ration will be kept."""
         self.gui_repaint()
 
 
-    def draw(self):
+    def draw(self, repaint=True):
         """
         Render the figure using RendererWx instance renderer, or using a
         previously defined renderer if none is specified.
@@ -877,7 +898,8 @@ The current aspect ration will be kept."""
         DEBUG_MSG("draw()", 1, self)
         self.renderer = RendererWx(self.bitmap, self.figure.dpi)
         self.figure.draw(self.renderer)
-        self.gui_repaint()
+        if repaint:
+            self.gui_repaint()
 
     def _get_imagesave_wildcards(self):
         'return the wildcard string for the filesave dialog'
@@ -890,23 +912,22 @@ The current aspect ration will be kept."""
                "PNG (*.png)|*.png|"  \
                "XPM (*.xpm)|*.xpm"
 
-    def gui_repaint(self):
+    def gui_repaint(self, drawDC=None):
         """
-        Performs update of the displayed image on the GUI canvas
-
-        MUST NOT be called during a Paint event
+        Performs update of the displayed image on the GUI canvas, using the
+        supplied device context.  If drawDC is None, a ClientDC will be used to
+        redraw the image.
         """
         DEBUG_MSG("gui_repaint()", 1, self)
-        drawDC=wx.ClientDC(self)
+        if drawDC is None:
+            drawDC=wx.ClientDC(self)
+
         drawDC.BeginDrawing()
-        #drawDC.Clear()
         drawDC.DrawBitmap(self.bitmap, 0, 0)
         drawDC.EndDrawing()
 
-    def print_figure(self, filename, dpi=150,
-                     facecolor='w', edgecolor='w',
-                     orientation='portrait'):
-
+    def print_figure(self, filename, dpi=150, facecolor='w', edgecolor='w',
+                     orientation='portrait', **kwargs):
         """
         Render the figure to hardcopy
         """
@@ -926,7 +947,8 @@ The current aspect ration will be kept."""
             ps = self.switch_backends(FigureCanvasPS)
             ps.figure.dpi.set(72)
 
-            ps.print_figure(filename, 72, facecolor, edgecolor)
+            ps.print_figure(filename, 72, facecolor, edgecolor, orientation, 
+                            **kwargs)
             self.figure.dpi.set(origDPI)
             self.figure.set_canvas(self)
             return
@@ -939,7 +961,8 @@ The current aspect ration will be kept."""
             origDPI = self.figure.dpi.get()
             svg = self.switch_backends(FigureCanvasSVG)
             svg.figure.dpi.set(72)
-            svg.print_figure(filename, 72, facecolor, edgecolor)
+            svg.print_figure(filename, 72, facecolor, edgecolor, orientation,
+                             **kwargs)
             self.figure.dpi.set(origDPI)
             self.figure.set_canvas(self)
             return
@@ -1022,27 +1045,9 @@ The current aspect ration will be kept."""
         if not self._isRealized:
             self.realize()
         # Render to the bitmap
-        self.draw()
-        # Must use wxPaintDC during paint event
-
-        l,b,w,h = self.figure.bbox.get_bounds()
-        w = int(math.ceil(w))
-        h = int(math.ceil(h))
-
-
-        # I decoupled this from GraphicsContextWx so it would play
-        # nice with wxagg
-        memDC =wx.MemoryDC()
-        memDC.SelectObject(self.bitmap)
-        memDC.SetPen(wx.Pen('BLACK', 1, wx.SOLID))
-
-        drawDC=wx.PaintDC(self)
-
-        drawDC.BeginDrawing()
-        drawDC.Clear()
-        drawDC.Blit(0, 0, w, h, memDC, 0, 0)
-        #drawDC.DrawBitmap(self.bitmap, 0, 0)
-        drawDC.EndDrawing()
+        self.draw(repaint=False)
+        # Update the display using a PaintDC
+        self.gui_repaint(drawDC=wx.PaintDC(self))
         evt.Skip()
 
     def _onSize(self, evt):
@@ -1067,7 +1072,7 @@ The current aspect ration will be kept."""
         dpival = self.figure.dpi.get()
         winch = self._width/dpival
         hinch = self._height/dpival
-        self.figure.set_figsize_inches(winch, hinch)
+        self.figure.set_size_inches(winch, hinch)
 
         if self._isRealized:
             self.draw()
@@ -1205,7 +1210,8 @@ def new_figure_manager(num, *args, **kwargs):
         wxapp = wx.PySimpleApp()
         wxapp.SetExitOnFrameDelete(True)
 
-    fig = Figure(*args, **kwargs)
+    FigureClass = kwargs.pop('FigureClass', Figure)
+    fig = FigureClass(*args, **kwargs)
     frame = FigureFrameWx(num, fig)
     figmgr = frame.get_figure_manager()
     if matplotlib.is_interactive():
@@ -1237,14 +1243,7 @@ class FigureFrameWx(wx.Frame):
         # By adding toolbar in sizer, we are able to put it at the bottom
         # of the frame - so appearance is closer to GTK version
 
-
-        if matplotlib.rcParams['toolbar']=='classic':
-            self.toolbar = NavigationToolbarWx(self.canvas, True)
-        elif matplotlib.rcParams['toolbar']=='toolbar2':
-            self.toolbar = NavigationToolbar2Wx(self.canvas)
-            self.toolbar.set_status_bar(statbar)
-        else:
-            self.toolbar = None
+        self.toolbar = self._get_toolbar(statbar)
 
         if self.toolbar is not None:
             self.toolbar.Realize()
@@ -1275,6 +1274,16 @@ class FigureFrameWx(wx.Frame):
             # Event handlers 2.4
             wx.EVT_CLOSE(self, self._onClose)
 
+    def _get_toolbar(self, statbar):
+        if matplotlib.rcParams['toolbar']=='classic':
+            toolbar = NavigationToolbarWx(self.canvas, True)
+        elif matplotlib.rcParams['toolbar']=='toolbar2':
+            toolbar = NavigationToolbar2Wx(self.canvas)
+            toolbar.set_status_bar(statbar)
+        else:
+            toolbar = None
+        return toolbar 
+        
     def get_canvas(self, fig):
         return FigureCanvasWx(self, -1, fig)
 
@@ -1315,8 +1324,9 @@ class FigureManagerWx(FigureManagerBase):
         FigureManagerBase.__init__(self, canvas, num)
         self.frame = frame
         self.window = frame
-        self.tb = frame.GetToolBar()
 
+        self.tb = frame.GetToolBar()
+        self.toolbar = self.tb  # consistent with other backends
         def notify_axes_change(fig):
             'this will be called whenever the current axes is changed'
             if self.tb != None: self.tb.update()
@@ -1346,6 +1356,7 @@ _NTB_Y_PAN_UP        =wx.NewId()
 _NTB_Y_PAN_DOWN      =wx.NewId()
 _NTB_Y_ZOOMIN        =wx.NewId()
 _NTB_Y_ZOOMOUT       =wx.NewId()
+#_NTB_SUBPLOT            =wx.NewId()
 _NTB_SAVE            =wx.NewId()
 _NTB_CLOSE           =wx.NewId()
 
@@ -1484,13 +1495,38 @@ cursord = {
     cursors.SELECT_REGION : wx.CURSOR_CROSS,
     }
 
+
+class SubplotToolWX(wx.Frame):
+    def __init__(self, targetfig):
+        wx.Frame.__init__(self, None, -1, "Configure subplots")
+
+        toolfig = Figure((6,3))
+        canvas = FigureCanvasWx(self, -1, toolfig)
+
+        # Create a figure manager to manage things
+        figmgr = FigureManager(canvas, 1, self)
+
+        # Now put all into a sizer
+        sizer = wx.BoxSizer(wx.VERTICAL)
+        # This way of adding to sizer allows resizing
+        sizer.Add(canvas, 1, wx.LEFT|wx.TOP|wx.GROW)
+        self.SetSizer(sizer)
+        self.Fit()
+        tool = SubplotTool(targetfig, toolfig)
+        
+
 class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
+
     def __init__(self, canvas):
         wx.ToolBar.__init__(self, canvas.GetParent(), -1)
         NavigationToolbar2.__init__(self, canvas)
         self.canvas = canvas
         self._idle = True
         self.statbar = None
+
+    def get_canvas(self, frame, fig):
+        return FigureCanvasWx(frame, -1, fig)
+        
     def _init_toolbar(self):
         DEBUG_MSG("_init_toolbar", 1, self)
 
@@ -1500,7 +1536,8 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
         self._NTB2_FORWARD =wx.NewId()
         self._NTB2_PAN     =wx.NewId()
         self._NTB2_ZOOM    =wx.NewId()
-        _NTB2_SAVE    =wx.NewId()
+        _NTB2_SAVE    = wx.NewId()
+        _NTB2_SUBPLOT    =wx.NewId()        
 
         self.SetToolBitmapSize(wx.Size(24,24))
 
@@ -1518,6 +1555,9 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
                            shortHelp='Zoom', longHelp='Zoom to rectangle')
 
         self.AddSeparator()
+        self.AddSimpleTool(_NTB2_SUBPLOT, _load_bitmap('subplots.xpm'),
+                           'Configure subplots', 'Configure subplot parameters')
+
         self.AddSimpleTool(_NTB2_SAVE, _load_bitmap('filesave.xpm'),
                            'Save', 'Save plot contents to file')
 
@@ -1527,6 +1567,7 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
             self.Bind(wx.EVT_TOOL, self.back, id=self._NTB2_BACK)
             self.Bind(wx.EVT_TOOL, self.zoom, id=self._NTB2_ZOOM)
             self.Bind(wx.EVT_TOOL, self.pan, id=self._NTB2_PAN)
+            self.Bind(wx.EVT_TOOL, self.configure_subplot, id=_NTB2_SUBPLOT)            
             self.Bind(wx.EVT_TOOL, self.save, id=_NTB2_SAVE)
         else:
             wx.EVT_TOOL(self, _NTB2_HOME, self.home)
@@ -1534,6 +1575,7 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
             wx.EVT_TOOL(self, self._NTB2_BACK, self.back)
             wx.EVT_TOOL(self, self._NTB2_ZOOM, self.zoom)
             wx.EVT_TOOL(self, self._NTB2_PAN, self.pan)
+            wx.EVT_TOOL(self, _NTB2_SUBPLOT, self.configure_subplot)            
             wx.EVT_TOOL(self, _NTB2_SAVE, self.save)
 
         self.Realize()
@@ -1547,6 +1589,25 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
         self.ToggleTool(self._NTB2_ZOOM, False)
         NavigationToolbar2.pan(self, *args)
 
+
+    def configure_subplot(self, evt):
+        frame = wx.Frame(None, -1, "Configure subplots")
+
+        toolfig = Figure((6,3))
+        canvas = self.get_canvas(frame, toolfig)
+
+        # Create a figure manager to manage things
+        figmgr = FigureManager(canvas, 1, frame)
+
+        # Now put all into a sizer
+        sizer = wx.BoxSizer(wx.VERTICAL)
+        # This way of adding to sizer allows resizing
+        sizer.Add(canvas, 1, wx.LEFT|wx.TOP|wx.GROW)
+        frame.SetSizer(sizer)
+        frame.Fit()
+        tool = SubplotTool(self.canvas.figure, toolfig)
+        frame.Show()
+        
     def save(self, evt):
         # Fetch the required filename and file type.
         filetypes = self.canvas._get_imagesave_wildcards()
