@@ -7,7 +7,7 @@ from matplotlib import verbose
 import matplotlib
 import math
 from artist import Artist
-from cbook import enumerate, popd, is_string_like
+from cbook import enumerate, popd, is_string_like, maxdict
 from font_manager import FontProperties
 from matplotlib import rcParams
 from patches import bbox_artist
@@ -49,14 +49,14 @@ def scanner(s):
     results = []
 
     inmath = s[0] == '$'
-    for i in range(Nkeep-1):        
+    for i in range(Nkeep-1):
         i0, i1 = indkeep[i], indkeep[i+1]
         if not inmath:
             if i0>0: i0 +=1
         else:
             i1 += 1
         if i0>=Ns: break
-            
+
         results.append((s[i0:i1], inmath))
         inmath = not inmath
 
@@ -78,12 +78,14 @@ def get_rotation(rotation):
     'return the text angle as float'
     if rotation in ('horizontal', None):
         angle = 0.
-    elif rotation == 'vertical':            
+    elif rotation == 'vertical':
         angle = 90.
     else:
         angle = float(rotation)
     return angle%360
 
+_unit_box = lbwh_to_bbox(0,0,1,1)
+
 class Text(Artist):
     """
     Handle storing and drawing of text in window or data coordinates
@@ -93,7 +95,7 @@ class Text(Artist):
     _rgxsuper = re.compile('\$([\-+0-9]+)\^\{(-?[0-9]+)\}\$')
 
     zorder = 3
-    def __init__(self, 
+    def __init__(self,
                  x=0, y=0, text='',
                  color=None,          # defaults to rc params
                  verticalalignment='bottom',
@@ -106,33 +108,35 @@ class Text(Artist):
         Artist.__init__(self)
         if not is_string_like(text):
             raise TypeError('text must be a string type')
-        self.cached = {}
+        self.cached = maxdict(5)
         self._x, self._y = x, y
 
         if color is None: color = rcParams['text.color']
         if fontproperties is None: fontproperties=FontProperties()
 
-        self._color = color
+        self.set_color(color)
         self.set_text(text)
         self._verticalalignment = verticalalignment
         self._horizontalalignment = horizontalalignment
-        self._multialignment = multialignment        
+        self._multialignment = multialignment
         self._rotation = rotation
         self._fontproperties = fontproperties
         self._bbox = None
-        self._renderer = None        
-        
+        self._renderer = None
+
+        #self.set_bbox(dict(pad=0))
+
     def _get_multialignment(self):
         if self._multialignment is not None: return self._multialignment
         else: return self._horizontalalignment
-    
+
     def get_rotation(self):
         'return the text angle as float'
         #return 0
 
 #         if self._rotation in ('horizontal', None):
 #             angle = 0.
-#         elif self._rotation == 'vertical':            
+#         elif self._rotation == 'vertical':
 #             angle = 90.
 #         else:
 #             angle = float(self._rotation)
@@ -142,12 +146,12 @@ class Text(Artist):
         # into a function for TextWithDash, this
         # method could now read as follows.
         return get_rotation(self._rotation)
-    
+
     def update_from(self, other):
-        'Copy properties from t to self'
+        'Copy properties from other to self'
         Artist.update_from(self, other)
         self._color = other._color
-        self._multialignment = other._multialignment        
+        self._multialignment = other._multialignment
         self._verticalalignment = other._verticalalignment
         self._horizontalalignment = other._horizontalalignment
         self._fontproperties = other._fontproperties.copy()
@@ -158,10 +162,9 @@ class Text(Artist):
 
         # layout the xylocs in display coords as if angle = zero and
         # then rotate them around self._x, self._y
-        
+        #return _unit_box
         key = self.get_prop_tup()
         if self.cached.has_key(key): return self.cached[key]
-        
         horizLayout = []
         pad =2
         thisx, thisy = self._transform.xy_tup( (self._x, self._y) )
@@ -175,30 +178,30 @@ class Text(Artist):
             lines = self._text.split('\n')
 
         whs = []
+        tmp, heightt = renderer.get_text_width_height(
+                'T', self._fontproperties, ismath=False)
+
+        heightt += 3  # 3 pixel pad
         for line in lines:
             w,h = renderer.get_text_width_height(
                 line, self._fontproperties, ismath=self.is_math_text())
-            if not len(line) and not self.is_math_text():
-                # approx the height of empty line with tall char
-                tmp, h = renderer.get_text_width_height(
-                'T', self._fontproperties, ismath=False)
 
             whs.append( (w,h) )
-            offsety = h+pad
+            offsety = heightt+pad
             horizLayout.append((line, thisx, thisy, w, h))
             thisy -= offsety  # now translate down by text height, window coords
             width = max(width, w)
-        
+
         ymin = horizLayout[-1][2]
         ymax = horizLayout[0][2] + horizLayout[0][-1]
         height = ymax-ymin
 
         xmax = xmin + width
         # get the rotation matrix
-        M = self.get_rotation_matrix(xmin, ymin) 
+        M = self.get_rotation_matrix(xmin, ymin)
 
         # the corners of the unrotated bounding box
-        cornersHoriz = ( (xmin, ymin), (xmin, ymax), (xmax, ymax), (xmax, ymin) )                
+        cornersHoriz = ( (xmin, ymin), (xmin, ymax), (xmax, ymax), (xmax, ymin) )
         offsetLayout = []
         # now offset the individual text lines within the box
         if len(lines)>1: # do the multiline aligment
@@ -213,7 +216,7 @@ class Text(Artist):
             offsetLayout = [ (thisx, thisy) for line, thisx, thisy, w, h in horizLayout]
 
         # now rotate the bbox
-        
+
         cornersRotated = [dot(M,array([[thisx],[thisy],[1]])) for thisx, thisy in cornersHoriz]
 
         txs = [float(v[0][0]) for v in cornersRotated]
@@ -227,12 +230,12 @@ class Text(Artist):
 
         # Now move the box to the targe position offset the display bbox by alignment
         halign = self._horizontalalignment
-        valign = self._verticalalignment        
+        valign = self._verticalalignment
 
         # compute the text location in display coords and the offsets
         # necessary to align the bbox with that location
         tx, ty = self._transform.xy_tup( (self._x, self._y) )
-        
+
         if halign=='center':  offsetx = tx - (xmin + width/2.0)
         elif halign=='right': offsetx = tx - (xmin + width)
         else: offsetx = tx - xmin
@@ -248,7 +251,7 @@ class Text(Artist):
 
         bbox = lbwh_to_bbox(xmin, ymin, width, height)
 
-        
+
         # now rotate the positions around the first x,y position
         xys = [dot(M,array([[thisx],[thisy],[1]])) for thisx, thisy in offsetLayout]
 
@@ -264,22 +267,23 @@ class Text(Artist):
         ret = bbox, zip(lines, whs, xs, ys)
         self.cached[key] = ret
         return ret
-         
+
 
     def set_bbox(self, rectprops):
         """
-Draw a bounding box around self.  rect props are any settable
-properties for a rectangle, eg color='r', alpha=0.5
+        Draw a bounding box around self.  rect props are any settable
+        properties for a rectangle, eg color='r', alpha=0.5
 
-ACCEPTS: rectangle prop dict plus key 'pad' which is a pad in points
+        ACCEPTS: rectangle prop dict plus key 'pad' which is a pad in points
         """
         self._bbox = rectprops
 
     def draw(self, renderer):
-        if not self.get_visible(): return 
-        if self._text=='': return
+        #return
         if renderer is not None:
             self._renderer = renderer
+        if not self.get_visible(): return
+        if self._text=='': return
 
         gc = renderer.new_gc()
         gc.set_foreground(self._color)
@@ -288,17 +292,18 @@ ACCEPTS: rectangle prop dict plus key 'pad' which is a pad in points
             gc.set_clip_rectangle(self.clipbox.get_bounds())
 
 
-            
+
         if self._bbox:
             bbox_artist(self, renderer, self._bbox)
         angle = self.get_rotation()
 
         ismath = self.is_math_text()
-        
+
         if angle==0:
-            if ismath!='TeX': m = None
+            #print 'text', self._text
+            if ismath=='TeX': m = None
             else: m = self._rgxsuper.match(self._text)
-            if m is not None and not rcParams['text.usetex']:
+            if m is not None:
                 bbox, info = self._get_layout_super(self._renderer, m)
                 base, xt, yt = info[0]
                 renderer.draw_text(gc, xt, yt, base,
@@ -323,13 +328,13 @@ ACCEPTS: rectangle prop dict plus key 'pad' which is a pad in points
                 if renderer.flipy():
                     canvasw, canvash = renderer.get_canvas_width_height()
                     rendery = canvash-rendery
-            
+
                 renderer.draw_text(gc, renderx, rendery, s,
                                    self._fontproperties, angle,
                                    ismath)
                 thisx += w
 
-                
+
             return
         bbox, info = self._get_layout(renderer)
 
@@ -339,10 +344,12 @@ ACCEPTS: rectangle prop dict plus key 'pad' which is a pad in points
                 x, y = self._transform.xy_tup((x, y))
                 if renderer.flipy():
                     y = canvash-y
+
                 renderer.draw_tex(gc, x, y, line,
                                   self._fontproperties, angle)
-            return 
-            
+            return
+
+        #print 'xy', self._x, self._y, info
         for line, wh, x, y in info:
             x, y = self._transform.xy_tup((x, y))
             #renderer.draw_arc(gc, (1,0,0),
@@ -351,14 +358,14 @@ ACCEPTS: rectangle prop dict plus key 'pad' which is a pad in points
             if renderer.flipy():
                 canvasw, canvash = renderer.get_canvas_width_height()
                 y = canvash-y
-            
+
             renderer.draw_text(gc, x, y, line,
                                self._fontproperties, angle,
                                ismath=self.is_math_text())
 
     def get_color(self):
         "Return the color of the text"
-        return self._color 
+        return self._color
 
     def get_font_properties(self):
         "Return the font object"
@@ -418,10 +425,11 @@ ACCEPTS: rectangle prop dict plus key 'pad' which is a pad in points
         need to know if the text has changed
         """
 
-        x, y = self._transform.xy_tup((self._x, self._y))
-        return (x, y, self._text, self._color,
+        return (self._x, self._y, self._text, self._color,
                 self._verticalalignment, self._horizontalalignment,
-                hash(self._fontproperties), self._rotation)
+                hash(self._fontproperties), self._rotation,
+                self._transform.as_vec6_val(),
+                )
 
     def get_text(self):
         "Get the text as string"
@@ -436,10 +444,12 @@ ACCEPTS: rectangle prop dict plus key 'pad' which is a pad in points
         return self._verticalalignment
 
     def get_window_extent(self, renderer=None):
+        #return _unit_box
+        if not self.get_visible(): return _unit_box
         if self._text == '':
             tx, ty = self._transform.xy_tup( (self._x, self._y) )
             return lbwh_to_bbox(tx,ty,0,0)
-        
+
         if renderer is not None:
             self._renderer = renderer
         if self._renderer is None:
@@ -447,14 +457,16 @@ ACCEPTS: rectangle prop dict plus key 'pad' which is a pad in points
 
         angle = self.get_rotation()
         if angle==0:
-            m = self._rgxsuper.match(self._text)
+            ismath = self.is_math_text()
+            if ismath=='TeX': m = None
+            else: m = self._rgxsuper.match(self._text)
             if m is not None:
                 bbox, tmp = self._get_layout_super(self._renderer, m)
                 return bbox
         bbox, info = self._get_layout(self._renderer)
         return bbox
 
-            
+
 
     def get_rotation_matrix(self, x0, y0):
 
@@ -462,67 +474,71 @@ ACCEPTS: rectangle prop dict plus key 'pad' which is a pad in points
         # translate x0,y0 to origin
         Torigin = array([ [1, 0, -x0],
                            [0, 1, -y0],
-                           [0, 0, 1  ]]) 
-        
+                           [0, 0, 1  ]])
+
         # rotate by theta
         R = array([ [cos(theta),  -sin(theta), 0],
                      [sin(theta), cos(theta), 0],
-                     [0,           0,          1]]) 
+                     [0,           0,          1]])
 
         # translate origin back to x0,y0
         Tback = array([ [1, 0, x0],
                          [0, 1, y0],
-                         [0, 0, 1  ]]) 
+                         [0, 0, 1  ]])
 
 
         return dot(dot(Tback,R), Torigin)
-        
+
     def set_backgroundcolor(self, color):
         """
-Set the background color of the text
+        Set the background color of the text
 
-ACCEPTS: any matplotlib color - see help(colors)
-"""
+        ACCEPTS: any matplotlib color - see help(colors)
+        """
         self._backgroundcolor = color
 
-        
+
     def set_color(self, color):
         """
-Set the foreground color of the text
+        Set the foreground color of the text
 
-ACCEPTS: any matplotlib color - see help(colors)
-"""
+        ACCEPTS: any matplotlib color - see help(colors)
+        """
+        # Make sure it is hashable, or get_prop_tup will fail.
+        try:
+            hash(color)
+        except TypeError:
+            color = tuple(color)
         self._color = color
 
     def set_ha(self, align):
         'alias for set_horizontalalignment'
         self.set_horizontalalignment(align)
 
-    def set_horizontalalignment(self, align):        
+    def set_horizontalalignment(self, align):
         """
-Set the horizontal alignment to one of
+        Set the horizontal alignment to one of
 
-ACCEPTS: [ 'center' | 'right' | 'left' ]
+        ACCEPTS: [ 'center' | 'right' | 'left' ]
         """
         legal = ('center', 'right', 'left')
         if align not in legal:
             raise ValueError('Horizontal alignment must be one of %s' % str(legal))
-        self._horizontalalignment = align     
+        self._horizontalalignment = align
 
     def set_ma(self, align):
         'alias for set_verticalalignment'
         self.set_multialignment(align)
-        
+
 
     def set_multialignment(self, align):
         """
-Set the alignment for multiple lines layout.  The layout of the
-bounding box of all the lines is determined bu the horizontalalignment
-and verticalalignment properties, but the multiline text within that
-box can be
-
-ACCEPTS: ['left' | 'right' | 'center' ]
+        Set the alignment for multiple lines layout.  The layout of the
+        bounding box of all the lines is determined bu the horizontalalignment
+        and verticalalignment properties, but the multiline text within that
+        box can be
 
+        ACCEPTS: ['left' | 'right' | 'center' ]
         """
         legal = ('center', 'right', 'left')
         if align not in legal:
@@ -531,25 +547,25 @@ ACCEPTS: ['left' | 'right' | 'center' ]
 
     def set_family(self, fontname):
         """
-Set the font family
+        Set the font family
 
-ACCEPTS: [ 'serif' | 'sans-serif' | 'cursive' | 'fantasy' | 'monospace' ]
+        ACCEPTS: [ 'serif' | 'sans-serif' | 'cursive' | 'fantasy' | 'monospace' ]
         """
         self._fontproperties.set_family(fontname)
 
     def set_variant(self, variant):
         """
-Set the font variant, eg, 
+        Set the font variant, eg,
 
-ACCEPTS: [ 'normal' | 'small-caps' ]
+        ACCEPTS: [ 'normal' | 'small-caps' ]
         """
         self._fontproperties.set_variant(variant)
 
     def set_name(self, fontname):
         """
-Set the font name, 
+        Set the font name,
 
-ACCEPTS: string eg, ['Sans' | 'Courier' | 'Helvetica' ...]
+        ACCEPTS: string eg, ['Sans' | 'Courier' | 'Helvetica' ...]
         """
         self._fontproperties.set_name(fontname)
 
@@ -559,9 +575,9 @@ ACCEPTS: string eg, ['Sans' | 'Courier' | 'Helvetica' ...]
 
     def set_style(self, fontstyle):
         """
-Set the font style
+        Set the font style
 
-ACCEPTS: [ 'normal' | 'italic' | 'oblique']
+        ACCEPTS: [ 'normal' | 'italic' | 'oblique']
         """
         self._fontproperties.set_style(fontstyle)
 
@@ -571,66 +587,64 @@ ACCEPTS: [ 'normal' | 'italic' | 'oblique']
 
     def set_size(self, fontsize):
         """
-Set the font size, eg, 8, 10, 12, 14...
+        Set the font size, eg, 8, 10, 12, 14...
 
-ACCEPTS: [ size in points | relative size eg 'smaller', 'x-large' ]
+        ACCEPTS: [ size in points | relative size eg 'smaller', 'x-large' ]
         """
-        self._fontproperties.set_size(fontsize)        
+        self._fontproperties.set_size(fontsize)
 
     def set_fontsize(self, fontsize):
         'alias for set_size'
         self._fontproperties.set_size(fontsize)
-        
+
     def set_fontweight(self, weight):
         'alias for set_weight'
         self._fontproperties.set_weight(weight)
 
     def set_weight(self, weight):
         """
-Set the font weight
+        Set the font weight
 
-ACCEPTS: [ 'normal' | 'bold' | 'heavy' | 'light' | 'ultrabold' | 'ultralight']
+        ACCEPTS: [ 'normal' | 'bold' | 'heavy' | 'light' | 'ultrabold' | 'ultralight']
         """
         self._fontproperties.set_weight(weight)
-        
+
     def set_position(self, xy):
         """
-Set the xy position of the text
+        Set the xy position of the text
 
-ACCEPTS: (x,y)
-"""
+        ACCEPTS: (x,y)
+        """
         self.set_x(xy[0])
         self.set_y(xy[1])
 
-    def set_x(self, x):        
+    def set_x(self, x):
         """
-Set the x position of the text
+        Set the x position of the text
 
-ACCEPTS: float
-"""
-        try: self._x.set(x)
-        except AttributeError: self._x = x
+        ACCEPTS: float
+        """
+        self._x = float(x)
 
 
     def set_y(self, y):
         """
-Set the y position of the text
+        Set the y position of the text
+
+        ACCEPTS: float
+        """
+        self._y = float(y)
+
 
-ACCEPTS: float
-"""
-        try: self._y.set(y)
-        except AttributeError: self._y = y
-        
-        
     def set_rotation(self, s):
         """
-Set the rotation of the text
+        Set the rotation of the text
 
-ACCEPTS: [ angle in degrees 'vertical' | 'horizontal'
-"""
+        ACCEPTS: [ angle in degrees 'vertical' | 'horizontal'
+        """
         self._rotation = s
-        
-        
+
+
 
     def set_va(self, align):
         'alias for set_verticalalignment'
@@ -638,22 +652,22 @@ ACCEPTS: [ angle in degrees 'vertical' | 'horizontal'
 
     def set_verticalalignment(self, align):
         """
-Set the vertical alignment
+        Set the vertical alignment
 
-ACCEPTS: [ 'center' | 'top' | 'bottom' ]
+        ACCEPTS: [ 'center' | 'top' | 'bottom' ]
         """
         legal = ('top', 'bottom', 'center')
         if align not in legal:
             raise ValueError('Vertical alignment must be one of %s' % str(legal))
 
         self._verticalalignment = align
-        
+
     def set_text(self, s):
         """
-Set the text string s
+        Set the text string s
 
-ACCEPTS: string
-"""
+        ACCEPTS: string
+        """
         if not is_string_like(s):
             raise TypeError("This doesn't look like a string: '%s'"%s)
         self._text = s
@@ -669,21 +683,21 @@ ACCEPTS: string
 
     def set_fontproperties(self, fp):
         """
-Set the font properties that control the text
+        Set the font properties that control the text
 
-ACCEPTS: a matplotlib.font_manager.FontProperties instance
-"""
+        ACCEPTS: a matplotlib.font_manager.FontProperties instance
+        """
         self._fontproperties = fp
 
-        
-        
-    
+
+
+
     def _get_layout_super(self, renderer, m):
         """
         a special case optimization if a log super and angle = 0
         Basically, mathtext is slow and we can do simple superscript layout "by hand"
         """
-        
+
         key = self.get_prop_tup()
         if self.cached.has_key(key): return self.cached[key]
 
@@ -716,19 +730,19 @@ ACCEPTS: a matplotlib.font_manager.FontProperties instance
         xe += xo
         ye += yo
         bbox = lbwh_to_bbox(xb, yb, w, h)
-        
+
         if renderer.flipy():
             canvasw, canvash = renderer.get_canvas_width_height()
             yb = canvash-yb
-            ye = canvash-ye            
-        
-            
+            ye = canvash-ye
+
+
         val = ( bbox, ((base, xb, yb), (exponent, xe, ye, fpexp)))
         self.cached[key] = val
 
         return val
 
-class TextWithDash(Artist):
+class _TextWithDash(Artist):
     """
     This is basically a Text with a dash (drawn with a Line2D)
     before/after it. It is intended to be a drop-in replacement
@@ -777,7 +791,7 @@ class TextWithDash(Artist):
     """
     __name__ = 'textwithdash'
 
-    def __init__(self, 
+    def __init__(self,
                  x=0, y=0, text='',
                  color=None,          # defaults to rc params
                  verticalalignment='center',
@@ -816,7 +830,7 @@ class TextWithDash(Artist):
         self._dashrotation = dashrotation
         self._dashpad = dashpad
         self._dashpush = dashpush
-        
+
     def __getattr__(self, name):
         """Delegate most things to self._mytext.
         """
@@ -869,7 +883,7 @@ class TextWithDash(Artist):
         dashpad = self.get_dashpad()
         dashpush = self.get_dashpush()
         transform = self.get_transform()
-        
+
         angle = get_rotation(dashrotation)
         theta = pi*(angle/180.0+dashdirection-1)
         cos_theta, sin_theta = cos(theta), sin(theta)
@@ -924,7 +938,7 @@ class TextWithDash(Artist):
         we = self._mytext.get_window_extent(renderer=renderer)
         self._window_extent = we.deepcopy()
         self._window_extent.update(((c1[0], c1[1]),), False)
-        
+
         # Finally, make text align center
         self._mytext.set_horizontalalignment('center')
         self._mytext.set_verticalalignment('center')
@@ -935,16 +949,16 @@ class TextWithDash(Artist):
         else:
             self.update_coords(renderer)
             return self._window_extent
-        
+
     def get_dashlength(self):
         return self._dashlength
 
     def set_dashlength(self, dl):
         """
-Set the length of the dash.
+        Set the length of the dash.
 
-ACCEPTS: float
-"""
+        ACCEPTS: float
+        """
         self._dashlength = dl
 
     def get_dashdirection(self):
@@ -952,13 +966,13 @@ ACCEPTS: float
 
     def set_dashdirection(self, dd):
         """
-Set the direction of the dash following the text.
-1 is before the text and 0 is after. The default
-is 0, which is what you'd want for the typical
-case of ticks below and on the left of the figure.
+        Set the direction of the dash following the text.
+        1 is before the text and 0 is after. The default
+        is 0, which is what you'd want for the typical
+        case of ticks below and on the left of the figure.
 
-ACCEPTS: int
-"""
+        ACCEPTS: int
+        """
         self._dashdirection = dd
 
     def get_dashrotation(self):
@@ -969,10 +983,10 @@ ACCEPTS: int
 
     def set_dashrotation(self, dr):
         """
-Set the rotation of the dash.
+        Set the rotation of the dash.
 
-ACCEPTS: float
-"""
+        ACCEPTS: float
+        """
         self._dashrotation = dr
 
     def get_dashpad(self):
@@ -980,12 +994,12 @@ ACCEPTS: float
 
     def set_dashpad(self, dp):
         """
-Set the "pad" of the TextWithDash, which
-is the extra spacing between the dash and
-the text, in canvas units.
+        Set the "pad" of the TextWithDash, which
+        is the extra spacing between the dash and
+        the text, in canvas units.
 
-ACCEPTS: float
-"""
+        ACCEPTS: float
+        """
         self._dashpad = dp
 
     def get_dashpush(self):
@@ -993,12 +1007,12 @@ ACCEPTS: float
 
     def set_dashpush(self, dp):
         """
-Set the "push" of the TextWithDash, which
-is the extra spacing between the beginning
-of the dash and the specified position.
+        Set the "push" of the TextWithDash, which
+        is the extra spacing between the beginning
+        of the dash and the specified position.
 
-ACCEPTS: float
-"""
+        ACCEPTS: float
+        """
         self._dashpush = dp
 
     def get_position(self):
@@ -1007,37 +1021,37 @@ ACCEPTS: float
 
     def set_position(self, xy):
         """
-Set the xy position of the TextWithDash.
+        Set the xy position of the TextWithDash.
 
-ACCEPTS: (x,y)
-"""
+        ACCEPTS: (x,y)
+        """
         self.set_x(xy[0])
         self.set_y(xy[1])
 
-    def set_x(self, x):        
+    def set_x(self, x):
         """
-Set the x position of the TextWithDash.
+        Set the x position of the TextWithDash.
 
-ACCEPTS: float
-"""
+        ACCEPTS: float
+        """
         try: self._x.set(x)
         except AttributeError: self._x = x
 
     def set_y(self, y):
         """
-Set the y position of the TextWithDash.
+        Set the y position of the TextWithDash.
 
-ACCEPTS: float
-"""
+        ACCEPTS: float
+        """
         try: self._y.set(y)
         except AttributeError: self._y = y
 
     def set_transform(self, t):
         """
-Set the Transformation instance used by this artist.
+        Set the Transformation instance used by this artist.
 
-ACCEPTS: a matplotlib.transform transformation instance
-"""
+        ACCEPTS: a matplotlib.transform transformation instance
+        """
         self._transform = t
         self._transformSet = True
         self._mytext.set_transform(t)
@@ -1046,13 +1060,315 @@ ACCEPTS: a matplotlib.transform transformation instance
     def get_figure(self):
         'return the figure instance'
         return self.figure
-    
+
     def set_figure(self, fig):
         """
-Set the figure instance the artist belong to.
+        Set the figure instance the artist belong to.
 
-ACCEPTS: a matplotlib.figure.Figure instance
+        ACCEPTS: a matplotlib.figure.Figure instance
         """
         self.figure = fig
         self._mytext.set_figure(fig)
         self.dashline.set_figure(fig)
+
+
+class TextWithDash(Text):
+    """
+    This is basically a Text with a dash (drawn with a Line2D)
+    before/after it. It is intended to be a drop-in replacement
+    for Text, and should behave identically to Text when
+    dashlength=0.0.
+
+    The dash always comes between the point specified by
+    set_position() and the text. When a dash exists, the
+    text alignment arguments (horizontalalignment,
+    verticalalignment) are ignored.
+
+    dashlength is the length of the dash in canvas units.
+    (default=0.0).
+
+    dashdirection is one of 0 or 1, where 0 draws the dash
+    after the text and 1 before.
+    (default=0).
+
+    dashrotation specifies the rotation of the dash, and
+    should generally stay None. In this case
+    self.get_dashrotation() returns self.get_rotation().
+    (I.e., the dash takes its rotation from the text's
+    rotation). Because the text center is projected onto
+    the dash, major deviations in the rotation cause
+    what may be considered visually unappealing results.
+    (default=None).
+
+    dashpad is a padding length to add (or subtract) space
+    between the text and the dash, in canvas units.
+    (default=3).
+
+    dashpush "pushes" the dash and text away from the point
+    specified by set_position() by the amount in canvas units.
+    (default=0)
+
+    NOTE: The alignment of the two objects is based on the
+    bbox of the Text, as obtained by get_window_extent().
+    This, in turn, appears to depend on the font metrics
+    as given by the rendering backend. Hence the quality
+    of the "centering" of the label text with respect to
+    the dash varies depending on the backend used.
+
+    NOTE2: I'm not sure that I got the get_window_extent()
+    right, or whether that's sufficient for providing the
+    object bbox.
+    """
+    __name__ = 'textwithdash'
+
+    def __init__(self,
+                 x=0, y=0, text='',
+                 color=None,          # defaults to rc params
+                 verticalalignment='center',
+                 horizontalalignment='center',
+                 multialignment=None,
+                 fontproperties=None, # defaults to FontProperties()
+                 rotation=None,
+                 dashlength=0.0,
+                 dashdirection=0,
+                 dashrotation=None,
+                 dashpad=3,
+                 dashpush=0,
+                 xaxis=True,
+                 ):
+
+        Text.__init__(self, x=x, y=y, text=text, color=color,
+                      verticalalignment=verticalalignment,
+                      horizontalalignment=horizontalalignment,
+                      multialignment=multialignment,
+                      fontproperties=fontproperties, rotation=rotation)
+
+        # The position (x,y) values for text and dashline
+        # are bogus as given in the instantiation; they will
+        # be set correctly by update_coords() in draw()
+
+        self.dashline = Line2D(xdata=(x, x),
+                               ydata=(y, y),
+                               color='k',
+                               linestyle='-')
+
+        self._dashx = float(x)
+        self._dashy = float(y)
+        self._dashlength = dashlength
+        self._dashdirection = dashdirection
+        self._dashrotation = dashrotation
+        self._dashpad = dashpad
+        self._dashpush = dashpush
+
+        #self.set_bbox(dict(pad=0))
+
+    def draw(self, renderer):
+        self.update_coords(renderer)
+        Text.draw(self, renderer)
+        if self.get_dashlength() > 0.0:
+            self.dashline.draw(renderer)
+
+    def update_coords(self, renderer):
+        """Computes the actual x,y coordinates for
+        text based on the input x,y and the
+        dashlength. Since the rotation is with respect
+        to the actual canvas's coordinates we need to
+        map back and forth.
+        """
+        dashx, dashy = self.get_position()
+        dashlength = self.get_dashlength()
+        # Shortcircuit this process if we don't have a dash
+        if dashlength == 0.0:
+            self._x, self._y = dashx, dashy
+            return
+
+        dashrotation = self.get_dashrotation()
+        dashdirection = self.get_dashdirection()
+        dashpad = self.get_dashpad()
+        dashpush = self.get_dashpush()
+
+        angle = get_rotation(dashrotation)
+        theta = pi*(angle/180.0+dashdirection-1)
+        cos_theta, sin_theta = cos(theta), sin(theta)
+
+        transform = self.get_transform()
+
+        # Compute the dash end points
+        # The 'c' prefix is for canvas coordinates
+        cxy = array(transform.xy_tup((dashx, dashy)))
+        cd = array([cos_theta, sin_theta])
+        c1 = cxy+dashpush*cd
+        c2 = cxy+(dashpush+dashlength)*cd
+
+        (x1, y1) = transform.inverse_xy_tup(tuple(c1))
+        (x2, y2) = transform.inverse_xy_tup(tuple(c2))
+        self.dashline.set_data((x1, x2), (y1, y2))
+
+        # We now need to extend this vector out to
+        # the center of the text area.
+        # The basic problem here is that we're "rotating"
+        # two separate objects but want it to appear as
+        # if they're rotated together.
+        # This is made non-trivial because of the
+        # interaction between text rotation and alignment -
+        # text alignment is based on the bbox after rotation.
+        # We reset/force both alignments to 'center'
+        # so we can do something relatively reasonable.
+        # There's probably a better way to do this by
+        # embedding all this in the object's transformations,
+        # but I don't grok the transformation stuff
+        # well enough yet.
+        we = Text.get_window_extent(self, renderer=renderer)
+        w, h = we.width(), we.height()
+        # Watch for zeros
+        if sin_theta == 0.0:
+            dx = w
+            dy = 0.0
+        elif cos_theta == 0.0:
+            dx = 0.0
+            dy = h
+        else:
+            tan_theta = sin_theta/cos_theta
+            dx = w
+            dy = w*tan_theta
+            if dy > h or dy < -h:
+                dy = h
+                dx = h/tan_theta
+        cwd = array([dx, dy])/2
+        cwd *= 1+dashpad/sqrt(dot(cwd,cwd))
+        cw = c2+(dashdirection*2-1)*cwd
+
+        self._x, self._y = transform.inverse_xy_tup(tuple(cw))
+
+        # Now set the window extent
+        # I'm not at all sure this is the right way to do this.
+        we = Text.get_window_extent(self, renderer=renderer)
+        self._twd_window_extent = we.deepcopy()
+        self._twd_window_extent.update(((c1[0], c1[1]),), False)
+
+        # Finally, make text align center
+        Text.set_horizontalalignment(self, 'center')
+        Text.set_verticalalignment(self, 'center')
+
+    def get_window_extent(self, renderer=None):
+        self.update_coords(renderer)
+        if self.get_dashlength() == 0.0:
+            return Text.get_window_extent(self, renderer=renderer)
+        else:
+            return self._twd_window_extent
+
+    def get_dashlength(self):
+        return self._dashlength
+
+    def set_dashlength(self, dl):
+        """
+        Set the length of the dash.
+
+        ACCEPTS: float
+        """
+        self._dashlength = dl
+
+    def get_dashdirection(self):
+        return self._dashdirection
+
+    def set_dashdirection(self, dd):
+        """
+        Set the direction of the dash following the text.
+        1 is before the text and 0 is after. The default
+        is 0, which is what you'd want for the typical
+        case of ticks below and on the left of the figure.
+
+        ACCEPTS: int
+        """
+        self._dashdirection = dd
+
+    def get_dashrotation(self):
+        if self._dashrotation == None:
+            return self.get_rotation()
+        else:
+            return self._dashrotation
+
+    def set_dashrotation(self, dr):
+        """
+        Set the rotation of the dash.
+
+        ACCEPTS: float
+        """
+        self._dashrotation = dr
+
+    def get_dashpad(self):
+        return self._dashpad
+
+    def set_dashpad(self, dp):
+        """
+        Set the "pad" of the TextWithDash, which
+        is the extra spacing between the dash and
+        the text, in canvas units.
+
+        ACCEPTS: float
+        """
+        self._dashpad = dp
+
+    def get_dashpush(self):
+        return self._dashpush
+
+    def set_dashpush(self, dp):
+        """
+        Set the "push" of the TextWithDash, which
+        is the extra spacing between the beginning
+        of the dash and the specified position.
+
+        ACCEPTS: float
+        """
+        self._dashpush = dp
+
+    def get_position(self):
+        "Return x, y as tuple"
+        return self._dashx, self._dashy
+
+    def set_position(self, xy):
+        """
+        Set the xy position of the TextWithDash.
+
+        ACCEPTS: (x,y)
+        """
+        self.set_x(xy[0])
+        self.set_y(xy[1])
+
+    def set_x(self, x):
+        """
+        Set the x position of the TextWithDash.
+
+        ACCEPTS: float
+        """
+        self._dashx = float(x)
+
+    def set_y(self, y):
+        """
+        Set the y position of the TextWithDash.
+
+        ACCEPTS: float
+        """
+        self._dashy = float(y)
+
+    def set_transform(self, t):
+        """
+        Set the Transformation instance used by this artist.
+
+        ACCEPTS: a matplotlib.transform transformation instance
+        """
+        Text.set_transform(self, t)
+        self.dashline.set_transform(t)
+
+    def get_figure(self):
+        'return the figure instance'
+        return self.figure
+
+    def set_figure(self, fig):
+        """
+        Set the figure instance the artist belong to.
+
+        ACCEPTS: a matplotlib.figure.Figure instance
+        """
+        Text.set_figure(self, fig)
+        self.dashline.set_figure(fig)
